@@ -25,6 +25,8 @@
 #include <glib/gi18n-lib.h>
 #include "application.h"
 #include "private.h"
+#include "wnck-handle-private.h"
+#include "wnck-icon-cache-private.h"
 
 /**
  * SECTION:application
@@ -48,8 +50,6 @@
 
 #define FALLBACK_NAME _("Untitled application")
 
-static GHashTable *app_hash = NULL;
-
 struct _WnckApplicationPrivate
 {
   Window xwindow; /* group leader */
@@ -72,7 +72,6 @@ struct _WnckApplicationPrivate
   char *startup_id;
 
   guint name_from_leader : 1; /* name is from group leader */
-  guint icon_from_leader : 1;
 
   guint need_emit_icon_changed : 1;
 };
@@ -95,16 +94,6 @@ static void wnck_application_finalize    (GObject        *object);
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
-void
-_wnck_application_shutdown_all (void)
-{
-  if (app_hash != NULL)
-    {
-      g_hash_table_destroy (app_hash);
-      app_hash = NULL;
-    }
-}
-
 static void
 wnck_application_init (WnckApplication *application)
 {
@@ -202,10 +191,7 @@ wnck_application_finalize (GObject *object)
 WnckApplication*
 wnck_application_get (gulong xwindow)
 {
-  if (app_hash == NULL)
-    return NULL;
-  else
-    return g_hash_table_lookup (app_hash, &xwindow);
+  return wnck_handle_get_application (_wnck_get_handle (), xwindow);
 }
 
 /**
@@ -327,15 +313,18 @@ wnck_application_get_pid (WnckApplication *app)
 static void
 get_icons (WnckApplication *app)
 {
+  WnckHandle *handle;
   GdkPixbuf *icon;
   GdkPixbuf *mini_icon;
   gsize normal_size;
   gsize mini_size;
 
+  handle = wnck_screen_get_handle (app->priv->screen);
+
   icon = NULL;
   mini_icon = NULL;
-  normal_size = _wnck_get_default_icon_size ();
-  mini_size = _wnck_get_default_mini_icon_size ();
+  normal_size = _wnck_handle_get_default_icon_size (handle);
+  mini_size = _wnck_handle_get_default_mini_icon_size (handle);
 
   if (_wnck_read_icons (app->priv->screen,
                         app->priv->xwindow,
@@ -346,7 +335,6 @@ get_icons (WnckApplication *app)
                         mini_size))
     {
       app->priv->need_emit_icon_changed = TRUE;
-      app->priv->icon_from_leader = TRUE;
 
       if (app->priv->icon)
         g_object_unref (G_OBJECT (app->priv->icon));
@@ -513,15 +501,14 @@ WnckApplication*
 _wnck_application_create (Window      xwindow,
                           WnckScreen *screen)
 {
+  WnckHandle      *handle;
   WnckApplication *application;
   Screen          *xscreen;
 
-  if (app_hash == NULL)
-    app_hash = g_hash_table_new_full (_wnck_xid_hash, _wnck_xid_equal,
-                                      NULL, g_object_unref);
+  handle = wnck_screen_get_handle (screen);
+  application = wnck_handle_get_application (handle, xwindow);
 
-  g_return_val_if_fail (g_hash_table_lookup (app_hash, &xwindow) == NULL,
-                        NULL);
+  g_return_val_if_fail (application == NULL, NULL);
 
   xscreen = WNCK_SCREEN_XSCREEN (screen);
 
@@ -544,9 +531,11 @@ _wnck_application_create (Window      xwindow,
                                                            application->priv->xwindow,
                                                            _wnck_atom_get ("_NET_STARTUP_ID"));
 
-  g_hash_table_insert (app_hash, &application->priv->xwindow, application);
+  _wnck_handle_insert_application (handle,
+                                   &application->priv->xwindow,
+                                   application);
 
-  /* Hash now owns one ref, caller gets none */
+  /* Handle now owns one ref, caller gets none */
 
   /* Note that xwindow may correspond to a WnckWindow's xwindow,
    * so we select events needed by either
@@ -562,15 +551,18 @@ _wnck_application_create (Window      xwindow,
 void
 _wnck_application_destroy (WnckApplication *application)
 {
+  WnckHandle *handle;
   Window xwindow = application->priv->xwindow;
 
-  g_return_if_fail (wnck_application_get (xwindow) == application);
+  handle = wnck_screen_get_handle (application->priv->screen);
 
-  g_hash_table_remove (app_hash, &xwindow);
+  g_return_if_fail (wnck_handle_get_application (handle, xwindow) == application);
 
-  /* Removing from hash also removes the only ref WnckApplication had */
+  _wnck_handle_remove_application (handle, &xwindow);
 
-  g_return_if_fail (wnck_application_get (xwindow) == NULL);
+  /* Removing from handle also removes the only ref WnckApplication had */
+
+  g_return_if_fail (wnck_handle_get_application (handle, xwindow) == NULL);
 }
 
 static void
@@ -658,8 +650,6 @@ _wnck_application_process_property_notify (WnckApplication *app,
   else if (xevent->xproperty.atom ==
            _wnck_atom_get ("_NET_WM_ICON") ||
            xevent->xproperty.atom ==
-           _wnck_atom_get ("KWM_WIN_ICON") ||
-           xevent->xproperty.atom ==
            _wnck_atom_get ("WM_NORMAL_HINTS"))
     {
       _wnck_icon_cache_property_changed (app->priv->icon_cache,
